/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2018 Synopsys, Inc. and/or its affiliates.
 * Synopsys DesignWare eDMA controller driver.
 */

#ifndef __PEDD_DFX_H__
#define __PEDD_DFX_H__


#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/string.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <asm/io.h>
#include "pedd_types.h"

#define MAX_MSG_SIZE        2048

#define WARNING_PRINT(fmt, args...) printk(KERN_WARNING "PEDD_WARNING [%s, %d]: " fmt, __FUNCTION__,  __LINE__, ## args)
#define ERROR_PRINT(fmt, args...) printk(KERN_ERR "PEDD_ERROR [%s, %d]: " fmt, __FUNCTION__,  __LINE__, ## args)

#ifdef PEDD_DEBUG_
#  define INFO_PRINT(fmt, args...) printk(KERN_DEBUG "PEDD_INFO [%s, %d]: " fmt, __FUNCTION__,  __LINE__, ## args)
#  define DTRACE(fmt,...) \
       do { printk(KERN_DEBUG "PEDD[%s, %d]: " fmt, \
    	    __FUNCTION__, __LINE__, ##__VA_ARGS__);} while(0)
#else
#  define INFO_PRINT(fmt, args...)
#  define DTRACE(fmt,...)
#endif

#define DPRINTF(...) \
    do {snprintf(__VA_ARGS__);} while(0)

#define DSTR_LEN(str,len) ((*len) = strlen(str))

#define DEPRINTF(des,s,fmt,...) \
    do {snprintf(des, s, "[ERR] " fmt, ##__VA_ARGS__);} while(0)

#define INT_SYNC_RUN(f,c) \
	do { \
		disable_irq((int)(c)->DmaRsc->Irq); \
		f(c); \
		enable_irq((int)(c)->DmaRsc->Irq); \
	}while(0)

#define KERNEL_WAIT(t) mdelay(t)
#define HW_WAIT() ndelay(1200)  // increased to 1200ns for Akida; otherwise missed
                                // interrupts may occur

#define RtlZeroMemory(Destination,Length) memset((Destination),0,(Length))
#define ZERO_MEM(p,s) RtlZeroMemory((p),(s))

#define PEDD_READ_UINT8(_addr)               ioread8((PUINT8)(_addr))
#define PEDD_READ_UINT16(_addr)              ioread16((PUINT16)(_addr))
#define PEDD_READ_UINT16_SAFE(_addr) \
	((UINT16)((UINT_PTR)_addr & 0x3 ? \
	ioread32((PUINT32)((UINT_PTR)_addr & (~((UINT_PTR)0) << 2))) >> 16  : \
	ioread32(_addr)))

#define PEDD_READ_UINT32(_addr)              ioread32((PUINT32)(_addr))
#define PEDD_WRITE_UINT8(_addr,Value)        iowrite8(Value, (PUINT8)(_addr))
#define PEDD_WRITE_UINT16(_addr,Value)       iowrite16(Value, (PUINT16)(_addr))
#define PEDD_WRITE_UINT16_SAFE(_addr,_Value) \
	do { \
		PUINT32 alignedAddr = (PUINT32)((UINT_PTR)_addr & (~((UINT_PTR)0) << 2)); \
		UINT32 oldv = ioread32(alignedAddr); \
		if ((char*)alignedAddr != (char*)_addr) { \
			iowrite32((oldv & 0xFFFF) | (_Value << 16), alignedAddr); \
		} else { \
			iowrite32((oldv & (0xFFFF << 16)) | _Value, alignedAddr); \
		} \
	} while(0)

#define PEDD_WRITE_UINT32(_addr,Value)       iowrite32(Value, (PUINT32)(_addr))

#define PEDD_DIV64(a, b) \
	({ \
		uint64_t t = a; \
		do_div(t, b); \
		(t); \
	})

#endif //__PEDD_DFX_H__
