/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2018 Synopsys, Inc. and/or its affiliates.
 * Synopsys DesignWare eDMA controller driver.
 */

#ifndef __PEDD_PERF_H__
#define __PEDD_PERF_H__

#include "pedd_types.h"

/******************************************************************************
 *
 *  DATA STRUCTURE DECLARATIONS
 *
 ******************************************************************************/

typedef struct PERF_REGS *PPERF_REGS;

typedef union {
    struct {
        // LSB
        UINT32      Rsvd0   :4; // 0 : 3
        UINT32      Reset   :1; // 4
        UINT32      Enb     :1; // 5
        UINT32      Rsvd1   :24;// 6 : 29
        UINT32      Tick    :2; // 30 : 31
        // MSB
    };
    UINT32 AsDword;
} PERF_CTRL;

typedef union {
    struct {
        // LSB
        UINT32      Low;        // 0 : 31
        UINT32      High;       // 32 : 63
        // MSB
    };
    UINT64 LargeInt;
} BIG_COUNTER;

typedef union {
    struct {
        // LSB
        UINT32      Low;        // 0 : 31
        UINT16      High;       // 32 : 47
        UINT16      Rsvd0;      // 48 : 63
        // MSB
    };
    UINT64 LargeInt;
} MID_COUNTER;

#pragma pack(push,1)
struct PERF_REGS {
    // LSB
    UINT32      Rsvd0[12];      // 0x00 - 0x2C
    BIG_COUNTER Time;           // 0x30 - 0x34
    MID_COUNTER RxTlp;          // 0x38 - 0x3C
    MID_COUNTER RxTlpNum;       // 0x40 - 0x44
    MID_COUNTER TxTlp;          // 0x48 - 0x4C
    MID_COUNTER TxTlpNum;       // 0x50 - 0x54
    PERF_CTRL   PerfCtrl;       // 0x58
    BIG_COUNTER RxDllp;         // 0x5C - 0x60
    BIG_COUNTER TxDllp;         // 0x64 - 0x68
    BIG_COUNTER RxSkipNum;      // 0x6C - 0x70
    BIG_COUNTER TxSkipNum;      // 0x74 - 0x78
    MID_COUNTER WrChanBytes[8]; // 0x7C - 0xB8
    MID_COUNTER RdChanBytes[8]; // 0xBC - 0xF8
    UINT32      Rsvd1;          // 0xFC
    MID_COUNTER XdWrBytes;      // 0x100 - 0x104
    MID_COUNTER XdRdBytes;      // 0x108 - 0x10C
    // MSB
};
#pragma pack(pop)


/******************************************************************************
 *
 *  EXTERN FUNCTION DECLARATIONS
 *
 ******************************************************************************/

void PerfReset(PPERF_REGS ppfreg);

void PerfSetOp(PPERF_REGS ppfreg, BOOLEAN enb);

BOOLEAN PerfGetData(PPERF_REGS ppfreg, PUINT64 time, PUCHAR tick,
        PUINT64 rxTlp, PUINT64 rxTlpNum, PUINT64 rxDllp, PUINT64 rxSkipNum,
        PUINT64 txTlp, PUINT64 txTlpNum, PUINT64 txDllp, PUINT64 txSkipNum,
        UINT64 *wrChanBytes, UINT64 *rdChanBytes,
        PUINT64 xdWrBytes, PUINT64 xdRdBytes);

#endif // __PEDD_PERF_H__
