// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2018 Synopsys, Inc. and/or its affiliates.
 * Synopsys DesignWare eDMA controller driver.
 */

#include "pedd_cmd.h"
#include "pedd_types.h"
#include <linux/string.h>

static lux_parse_cmd_t pedd_cb_args[PEDD_CB_NUM_ARGS] = {
    { "val",    0 },
    { "reg",    0 },
    { "bar",    0 },
    { "offset", 0 },
    { "size",   4 },
    { "chan",   0 },
    { "len",    512 },
    { "check",  0 },
    { "lle",    0 },
    { "dirc",   0 },
    { "lb",     0 },
    { "recyc",  0 },
    { "wm",  	0 },
    { "q",   	0 },
    { "ddr",   	0 },
    { "end",   	0 },
    { "wei",   	1 },
    { "up",   	1 }
};

static lux_parse_t pedd_cb = {
   PEDD_CB_NUM_ARGS,
   pedd_cb_args,
};

int pedd_parse_cmd(char *cmdline, char *cmd, unsigned long *arg_vals)
{
	return lux_parse_cmd_args(&pedd_cb, cmdline, cmd, arg_vals);
}

int lux_parse_cmd_args(lux_parse_t *arg_set, char *cmdline, char *cmd,
		unsigned long *arg_vals)
{
	int status = 0;
	char *end;
	char *argname;
	char *cp;
	int DONE;
	int FOUND = 0;

	unsigned short base;
	unsigned long result, value;
	unsigned long val;
	unsigned long i;
	unsigned long j;

	/*
	 * Init arg_vals with default values.
	 */
	for (i = 0; i < arg_set->ncmds; i++) {
		arg_vals[i] = arg_set->args[i].default_val;
	}

	/* get cmd */
	while (*cmdline == ' ' || *cmdline == '\t')
		cmdline++;

	while (*cmdline != ' ' && *cmdline != '\t' && *cmdline != '\0') {
		*cmd = *cmdline;
		cmd++;
		cmdline++;
	}
	*cmd = '\0';
	if (*cmdline == '\0') {
		goto WEDONE;
	}

	*cmdline = '\0';
	cmdline++;
	while (*cmdline == ' ' || *cmdline == '\t')
		cmdline++;

	end = cmdline;
	while (*end == ' ' || *end == '\t')
		end++;

	/*
	 * Parse cmdline
	 */
	DONE = (*end == '\0') ? 1 : 0;
	while (!DONE) {
		/* get the register name */
		while (*end != '=' && *end != '\0')
			end++;
		if (*end == '\0') {
			status = 1;
			goto WEDONE;
		}
		*end = '\0';
		argname = cmdline;

		/* now get value to write to register */
		cmdline = ++end;
		/* if there's whitespace after the '=', exit with an error */
		if (*end == ' ' || *end == '\t' || *end == '\n') {
			status = 1;
			goto WEDONE;
		}
		while (*end != ' ' && *end != '\t' && *end != '\n' && *end != '\0')
			end++;
		if (*end == '\0')
			DONE = 1;
		else
			*end = '\0';

		if (!strcmp(argname, "file") || !strcmp(argname, "filec")) {
			val = 1;
		} else {
			/* get the base, convert value to base-10 if necessary */
			val = 0;
			result = 0;
			cp = cmdline;
			if (cp[0] == '0' && (cp[1] == 'x' || cp[1] == 'X')) {
				base = 16;
				cp += 2;

			} else {
				base = 10;
			}
			/* val = simple_strtoul(cmdline, NULL, base); */
			while (isxdigit(*cp)) {
				value = isdigit(*cp) ? (*cp - '0')
						: ((islower(*cp) ? toupper(*cp) : *cp) - 'A' + 10);

				result = result * base + value;
				cp++;
			}

			val = result;

#if 0
			/* check that this value isn't too big for the registers;
			 they're 8 bits, so their max size is 255 */
			if (val > 255) {
				status = 1;
				goto WEDONE;
			}
#endif

		}

		FOUND = 0;
		/* verify the register name is valid, and if the value is not
		 too big, write it to the corresponding location in arg_vals */
		for (j = 0; j < arg_set->ncmds && !FOUND; j++) {
			if (!strcmp(argname, arg_set->args[j].arg)) {
				arg_vals[j] = val;
				FOUND = 1;
			}
		}
		if (!FOUND) {
			status = 1;
			goto WEDONE;
		}

		/* point cmdline and end to next non-whitespace (next argument) */
		cmdline = ++end;
		while (*cmdline == ' ' || *cmdline == '\t' || *cmdline == '\n')
			cmdline++;
		end = cmdline;
		/* if, after skipping whitespace, we hit end of line or EOF, we're done */
		if (*end == '\0')
			DONE = 1;
	}

WEDONE:
	return status;
}


