// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2018 Synopsys, Inc. and/or its affiliates.
 * Synopsys DesignWare eDMA controller driver.
 */

/*******************************************************************************
 *
 *  Header File Includes
 *
 ******************************************************************************/

#include "pedd_dfx.h"
#include "pedd_perf.h"

void PerfReset(PPERF_REGS ppfreg) {

    PERF_CTRL ctrl = {{0}};
    ctrl.Reset = 1;

    PEDD_WRITE_UINT32(&ppfreg->PerfCtrl, ctrl.AsDword);
}


void PerfSetOp(PPERF_REGS ppfreg, BOOLEAN enb) {

    PERF_CTRL ctrl = {{0}};
    ctrl.Enb = enb;

    PEDD_WRITE_UINT32(&ppfreg->PerfCtrl, ctrl.AsDword);
}

BOOLEAN PerfGetData(PPERF_REGS ppfreg, PUINT64 time, PUCHAR tick,
        PUINT64 rxTlp, PUINT64 rxTlpNum, PUINT64 rxDllp, PUINT64 rxSkipNum,
        PUINT64 txTlp, PUINT64 txTlpNum, PUINT64 txDllp, PUINT64 txSkipNum,
        UINT64 *wrChanBytes, UINT64 *rdChanBytes,
        PUINT64 xdWrBytes, PUINT64 xdRdBytes) {

    PERF_CTRL pctrl = {{0}};
    BIG_COUNTER timeReg = {{0}};
    MID_COUNTER rxTlpReg = {{0}};
    MID_COUNTER rxTlpNumReg = {{0}};
    BIG_COUNTER rxDllpReg = {{0}};
    BIG_COUNTER rxSkipNumReg = {{0}};
    MID_COUNTER txTlpReg = {{0}};
    MID_COUNTER txTlpNumReg = {{0}};
    BIG_COUNTER txDllpReg = {{0}};
    BIG_COUNTER txSkipNumReg = {{0}};
    MID_COUNTER chanBytesReg = {{0}};
    MID_COUNTER xdWrBytesReg = {{0}};
    MID_COUNTER xdRdBytesReg = {{0}};
    int i;

    /* Pause the counter */
    PerfSetOp(ppfreg, false);

    timeReg.Low = PEDD_READ_UINT32(&ppfreg->Time.Low);
    timeReg.High = PEDD_READ_UINT32(&ppfreg->Time.High);
    *time = jiffies;

    pctrl.AsDword = PEDD_READ_UINT32(&ppfreg->PerfCtrl);
    *tick = pctrl.Tick;

    rxTlpReg.Low = PEDD_READ_UINT32(&ppfreg->RxTlp.Low);
    rxTlpReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->RxTlp.High);
    *rxTlp = rxTlpReg.LargeInt;

    rxTlpNumReg.Low = PEDD_READ_UINT32(&ppfreg->RxTlpNum.Low);
    rxTlpNumReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->RxTlpNum.High);
    *rxTlpNum = rxTlpNumReg.LargeInt;

    rxDllpReg.Low = PEDD_READ_UINT32(&ppfreg->RxDllp.Low);
    rxDllpReg.High = PEDD_READ_UINT32(&ppfreg->RxDllp.High);
    *rxDllp = rxDllpReg.LargeInt;

    rxSkipNumReg.Low = PEDD_READ_UINT32(&ppfreg->RxSkipNum.Low);
    rxSkipNumReg.High = PEDD_READ_UINT32(&ppfreg->RxSkipNum.High);
    *rxSkipNum = rxSkipNumReg.LargeInt;

    txTlpReg.Low = PEDD_READ_UINT32(&ppfreg->TxTlp.Low);
    txTlpReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->TxTlp.High);
    *txTlp = txTlpReg.LargeInt;

    txTlpNumReg.Low = PEDD_READ_UINT32(&ppfreg->TxTlpNum.Low);
    txTlpNumReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->TxTlpNum.High);
    *txTlpNum = txTlpNumReg.LargeInt;

    txDllpReg.Low = PEDD_READ_UINT32(&ppfreg->TxDllp.Low);
    txDllpReg.High = PEDD_READ_UINT32(&ppfreg->TxDllp.High);
    *txDllp = txDllpReg.LargeInt;

    txSkipNumReg.Low = PEDD_READ_UINT32(&ppfreg->TxSkipNum.Low);
    txSkipNumReg.High = PEDD_READ_UINT32(&ppfreg->TxSkipNum.High);
    *txSkipNum = txSkipNumReg.LargeInt;

    for (i = 0; i < 8; i++) {
        chanBytesReg.Low = PEDD_READ_UINT32(&ppfreg->WrChanBytes[i].Low);
        chanBytesReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->WrChanBytes[i].High);
        wrChanBytes[i] = chanBytesReg.LargeInt;

        chanBytesReg.Low = PEDD_READ_UINT32(&ppfreg->RdChanBytes[i].Low);
        chanBytesReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->RdChanBytes[i].High);
        rdChanBytes[i] = chanBytesReg.LargeInt;
    }

    xdWrBytesReg.Low = PEDD_READ_UINT32(&ppfreg->XdWrBytes.Low);
    xdWrBytesReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->XdWrBytes.High);
    *xdWrBytes = xdWrBytesReg.LargeInt;

    xdRdBytesReg.Low = PEDD_READ_UINT32(&ppfreg->XdRdBytes.Low);
    xdRdBytesReg.High = PEDD_READ_UINT16_SAFE(&ppfreg->XdRdBytes.High);
    *xdRdBytes = xdRdBytesReg.LargeInt;

    /* Resume the counter */
    PerfSetOp(ppfreg, true);

    return TRUE;
}


